
#include <linux/module.h>
#include <linux/init.h>
#include <linux/fs.h>
#include <linux/miscdevice.h>
#include <asm/uaccess.h>


struct mv_gpio_ioctl {
	uint32 mask;
	uint32 val;
};

#define WPS_LED_IOCTL_NUM   'W'

/* reserve/release a gpio to the caller */
#define  LED_BLINK_WPS_ON   _IOWR(WPS_LED_IOCTL_NUM, 1, int *)
#define  LED_BLINK_WPS_OFF  _IOWR(WPS_LED_IOCTL_NUM, 2, int *)


/* major number assigned to the device and device handles */
static int gpio_major;
devfs_handle_t gpiodev_handle;

static int
mv_gpio_open(struct inode *inode, struct file * file)
{
	MOD_INC_USE_COUNT;
	return 0;
}

static int
mv_gpio_close(struct inode *inode, struct file * file)
{
	MOD_DEC_USE_COUNT;
	return 0;
}

static int
mv_gpio_ioctl(struct inode *inode, struct file *file, unsigned int cmd, unsigned long arg)
{
	struct mv_gpio_ioctl gpioioc;

	if (copy_from_user(&gpioioc, (struct mv_gpio_ioctl *)arg, sizeof(struct mv_gpio_ioctl)))
		return -EFAULT;

	switch (cmd) {
		case LED_BLINK_WPS:
		    break;
		default:
			break;
	}
	if (copy_to_user((struct mv_gpio_ioctl *)arg, &gpioioc, sizeof(struct mv_gpio_ioctl)))
		return -EFAULT;

	return 0;

}
static struct file_operations gpio_fops = {
	owner:		THIS_MODULE,
	open:		mv_gpio_open,
	release:	mv_gpio_close,
	ioctl:		mv_gpio_ioctl,
	};

static int __init
mv_gpio_init(void)
{
	if ((gpio_major = devfs_register_chrdev(0, "gpio", &gpio_fops)) < 0)
		return gpio_major;

	gpiodev_handle = devfs_register(NULL, "gpio", DEVFS_FL_DEFAULT,
	                                gpio_major, 0, S_IFCHR | S_IRUGO | S_IWUGO,
	                                &gpio_fops, NULL);

	return 0;
}

static void __exit
mv_gpio_exit(void)
{
	if (gpiodev_handle != NULL)
		devfs_unregister(gpiodev_handle);
	gpiodev_handle = NULL;
	devfs_unregister_chrdev(gpio_major, "gpio");
}

module_init(mv_gpio_init);
module_exit(mv_gpio_exit);
