#include <linux/kernel.h>
#include <linux/slab.h>
#include <linux/time.h>
#include "dbg-trace.h"

#define TRACE_ARR_LEN   2000
#define STR_LEN         30

struct trace {
	struct timeval tv;
        char str[STR_LEN];
	unsigned int callback_val1;
	unsigned int callback_val2;
        char valid;
};

static unsigned int (*trc_callback1) (unsigned char) = NULL;
static unsigned int (*trc_callback2) (unsigned char) = NULL;
static unsigned char trc_param1 = 0;
static unsigned char trc_param2 = 0;
struct trace *trc_arr;
static int trc_index;
static int output_flag = 0;

void TRC_INIT(void *callback1, void *callback2, unsigned char callback1_param, unsigned char callback2_param)
{
	printk("Marvell debug tracing is on\n");
        trc_arr = (struct trace *)kmalloc(TRACE_ARR_LEN*sizeof(struct trace),GFP_KERNEL);
	if (trc_arr == NULL) {
		printk("Error allocating TRC array\n");
		return;
	}
        memset(trc_arr,0,TRACE_ARR_LEN*sizeof(struct trace));
        trc_index = 0;
	trc_callback1 = callback1;
	trc_callback2 = callback2;
	trc_param1 = callback1_param;
	trc_param2 = callback2_param;
	output_flag = 0;
}

void TRC_REC(char *fmt,...)
{
	va_list args;
        struct trace *trc = &trc_arr[trc_index];
	if (output_flag)
		return;
	do_gettimeofday(&trc->tv);
	if(trc_callback1)
	    trc->callback_val1 = trc_callback1(trc_param1);
	if(trc_callback2)
	    trc->callback_val2 = trc_callback2(trc_param2);
	va_start(args, fmt);
	vsprintf(trc->str,fmt,args);
	va_end(args);
        trc->valid = 1;
        if((++trc_index) == TRACE_ARR_LEN) {
                trc_index = 0;
	}
}

void TRC_OUTPUT(void)
{
        int i,j;
        struct trace *p;
	output_flag = 1;
        printk("\n\nTrace %d items (trc_index=%d)\n",TRACE_ARR_LEN,trc_index);
        for(i=0,j=trc_index; i<TRACE_ARR_LEN; i++,j++) {
                if(j == TRACE_ARR_LEN) {
			printk("TRC_REC WRAPPING\n");
                        j = 0;
		}
                p = &trc_arr[j];
                if(p->valid) {
			unsigned long uoffs;
			struct trace *plast;
			if(p == &trc_arr[0])
				plast = &trc_arr[TRACE_ARR_LEN-1];
			else
				plast = p-1;
			if(p->tv.tv_sec == ((plast)->tv.tv_sec))
				uoffs = (p->tv.tv_usec - ((plast)->tv.tv_usec));
			else
				uoffs = (1000000 - ((plast)->tv.tv_usec)) +
					((p->tv.tv_sec - ((plast)->tv.tv_sec)) * 1000000) + 
					p->tv.tv_usec;
                        printk("[+%ld usec]",(unsigned long)uoffs);
			if(trc_callback1)
			    printk("[%u]",p->callback_val1);
			if(trc_callback2)
			    printk("[%u]",p->callback_val2);
			printk(": %s",p->str);
		}
                p->valid = 0;
        }
	output_flag = 0;
}

void TRC_RELEASE(void)
{
        kfree(trc_arr);
        trc_index = 0;
}


